@@ -1,12 +1,17 @@
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
-import re
-import six
+import decimal
 import math
+import re
+import typing
+
+from . import types
 
 
-def to_int(input_, default=0, exception=(ValueError, TypeError), regexp=None):
+def to_int(
+        input_: typing.Optional[str] = None,
+        default: int = 0,
+        exception: types.ExceptionsType = (ValueError, TypeError),
+        regexp: types.O[types.Pattern] = None,
+) -> int:
     r'''
     Convert the given input to an integer or return default
 
@@ -25,6 +30,10 @@ def to_int(input_, default=0, exception=(ValueError, TypeError), regexp=None):
     0
     >>> to_int('1')
     1
+    >>> to_int('')
+    0
+    >>> to_int()
+    0
     >>> to_int('abc123')
     0
     >>> to_int('123abc')
@@ -64,10 +73,9 @@ def to_int(input_, default=0, exception=(ValueError, TypeError), regexp=None):
     ...
     TypeError: unknown argument for regexp parameter: 123
     '''
-
     if regexp is True:
         regexp = re.compile(r'(\d+)')
-    elif isinstance(regexp, six.string_types):
+    elif isinstance(regexp, str):
         regexp = re.compile(regexp)
     elif hasattr(regexp, 'search'):
         pass
@@ -79,13 +87,21 @@ def to_int(input_, default=0, exception=(ValueError, TypeError), regexp=None):
             match = regexp.search(input_)
             if match:
                 input_ = match.groups()[-1]
-        return int(input_)
-    except exception:
+
+        if input_ is None:
+            return default
+        else:
+            return int(input_)
+    except exception:  # type: ignore
         return default
 
 
-def to_float(input_, default=0, exception=(ValueError, TypeError),
-             regexp=None):
+def to_float(
+        input_: str,
+        default: int = 0,
+        exception: types.ExceptionsType = (ValueError, TypeError),
+        regexp: types.O[types.Pattern] = None,
+) -> types.Number:
     r'''
     Convert the given `input_` to an integer or return default
 
@@ -140,7 +156,7 @@ def to_float(input_, default=0, exception=(ValueError, TypeError),
 
     if regexp is True:
         regexp = re.compile(r'(\d+(\.\d+|))')
-    elif isinstance(regexp, six.string_types):
+    elif isinstance(regexp, str):
         regexp = re.compile(regexp)
     elif hasattr(regexp, 'search'):
         pass
@@ -157,7 +173,11 @@ def to_float(input_, default=0, exception=(ValueError, TypeError),
         return default
 
 
-def to_unicode(input_, encoding='utf-8', errors='replace'):
+def to_unicode(
+        input_: types.StringTypes,
+        encoding: str = 'utf-8',
+        errors: str = 'replace',
+) -> str:
     '''Convert objects to unicode, if needed decodes string with the given
     encoding and errors settings.
 
@@ -175,14 +195,18 @@ def to_unicode(input_, encoding='utf-8', errors='replace'):
     >>> to_unicode(Foo)
     "<class 'python_utils.converters.Foo'>"
     '''
-    if isinstance(input_, six.binary_type):
+    if isinstance(input_, bytes):
         input_ = input_.decode(encoding, errors)
     else:
-        input_ = six.text_type(input_)
+        input_ = str(input_)
     return input_
 
 
-def to_str(input_, encoding='utf-8', errors='replace'):
+def to_str(
+        input_: types.StringTypes,
+        encoding: str = 'utf-8',
+        errors: str = 'replace',
+) -> bytes:
     '''Convert objects to string, encodes to the given encoding
 
     :rtype: str
@@ -199,17 +223,19 @@ def to_str(input_, encoding='utf-8', errors='replace'):
     >>> to_str(Foo)
     "<class 'python_utils.converters.Foo'>"
     '''
-    if isinstance(input_, six.binary_type):
+    if isinstance(input_, bytes):
         pass
     else:
         if not hasattr(input_, 'encode'):
-            input_ = six.text_type(input_)
+            input_ = str(input_)
 
         input_ = input_.encode(encoding, errors)
     return input_
 
 
-def scale_1024(x, n_prefixes):
+def scale_1024(
+        x: types.Number, n_prefixes: int,
+) -> types.Tuple[types.Number, types.Number]:
     '''Scale a number down to a suitable size, based on powers of 1024.
 
     Returns the scaled number and the power of 1024 used.
@@ -235,8 +261,12 @@ def scale_1024(x, n_prefixes):
     return scaled, power
 
 
-def remap(value, old_min, old_max, new_min, new_max):
-    """
+def remap(
+        value: types.DecimalNumber,
+        old_min: types.DecimalNumber, old_max: types.DecimalNumber,
+        new_min: types.DecimalNumber, new_max: types.DecimalNumber,
+) -> types.DecimalNumber:
+    '''
     remap a value from one range into another.
 
     >>> remap(500, 0, 1000, 0, 100)
@@ -247,6 +277,8 @@ def remap(value, old_min, old_max, new_min, new_max):
     -750
     >>> remap(33, 0, 100, -500, 500)
     -170
+    >>> remap(decimal.Decimal('250.0'), 0.0, 1000.0, 0.0, 100.0)
+    Decimal('25.0')
 
     This is a great use case example. Take an AVR that has dB values the
     minimum being -80dB and the maximum being 10dB and you want to convert
@@ -255,46 +287,102 @@ def remap(value, old_min, old_max, new_min, new_max):
     >>> remap(46.0, 0.0, 100.0, -80.0, 10.0)
     -38.6
 
+    I added using decimal.Decimal so floating point math errors can be avoided.
+    Here is an example of a floating point math error
+    >>> 0.1 + 0.1 + 0.1
+    0.30000000000000004
+
+    If floating point remaps need to be done my suggstion is to pass at least
+    one parameter as a `decimal.Decimal`. This will ensure that the output
+    from this function is accurate. I left passing `floats` for backwards
+    compatability and there is no conversion done from float to
+    `decimal.Decimal` unless one of the passed parameters has a type of
+    `decimal.Decimal`. This will ensure that any existing code that uses this
+    funtion will work exactly how it has in the past.
+
     Some edge cases to test
-    >>> remap(0, 0, 0, 0, 0)
-    0
-    >>> remap(0, 0, 0, 1, 0)
-    1
+    >>> remap(1, 0, 0, 1, 2)
+    Traceback (most recent call last):
+    ...
+    ValueError: Input range (0-0) is empty
+
+    >>> remap(1, 1, 2, 0, 0)
+    Traceback (most recent call last):
+    ...
+    ValueError: Output range (0-0) is empty
 
     :param value: value to be converted
-    :type value: int, float
+    :type value: int, float, decimal.Decimal
 
     :param old_min: minimum of the range for the value that has been passed
-    :type old_min: int, float
+    :type old_min: int, float, decimal.Decimal
 
     :param old_max: maximum of the range for the value that has been passed
-    :type old_max: int, float
+    :type old_max: int, float, decimal.Decimal
 
     :param new_min: the minimum of the new range
-    :type new_min: int, float
+    :type new_min: int, float, decimal.Decimal
 
     :param new_max: the maximum of the new range
-    :type new_max: int, float
+    :type new_max: int, float, decimal.Decimal
 
-    :return: value that has been re ranged, if the value is an int floor
-             division is used so the returned value will always be rounded down
-             to the closest whole number.
-    :rtype: int, float
-    """
-    old_range = old_max - old_min
-    new_range = new_max - new_min
-    if new_range == 0:
-        return 0
+    :return: value that has been re ranged. if any of the parameters passed is
+        a `decimal.Decimal` all of the parameters will be converted to
+        `decimal.Decimal`.  The same thing also happens if one of the
+        parameters is a `float`. otherwise all parameters will get converted
+        into an `int`. technically you can pass a `str` of an integer and it
+        will get converted. The returned value type will be `decimal.Decimal`
+        of any of the passed parameters ar `decimal.Decimal`, the return type
+        will be `float` if any of the passed parameters are a `float` otherwise
+        the returned type will be `int`.
+
+    :rtype: int, float, decimal.Decimal
+    '''
+    type_: types.Type[types.DecimalNumber]
+    if (
+            isinstance(value, decimal.Decimal) or
+            isinstance(old_min, decimal.Decimal) or
+            isinstance(old_max, decimal.Decimal) or
+            isinstance(new_min, decimal.Decimal) or
+            isinstance(new_max, decimal.Decimal)
+    ):
+        type_ = decimal.Decimal
+    elif (
+            isinstance(value, float) or
+            isinstance(old_min, float) or
+            isinstance(old_max, float) or
+            isinstance(new_min, float) or
+            isinstance(new_max, float)
+    ):
+        type_ = float
+
+    else:
+        type_ = int
+
+    value = type_(value)
+    old_min = type_(old_min)
+    old_max = type_(old_max)
+    new_max = type_(new_max)
+    new_min = type_(new_min)
+
+    old_range = old_max - old_min  # type: ignore
+    new_range = new_max - new_min  # type: ignore
 
     if old_range == 0:
-        new_value = new_min
-    else:
-        new_value = (value - old_min) * new_range
-        if isinstance(value, int):
-            new_value = new_value // old_range
-        else:
-            new_value = new_value / old_range
+        raise ValueError('Input range ({}-{}) is empty'.format(
+            old_min, old_max))
 
-        new_value += new_min
+    if new_range == 0:
+        raise ValueError('Output range ({}-{}) is empty'.format(
+            new_min, new_max))
+
+    new_value = (value - old_min) * new_range  # type: ignore
+
+    if type_ == int:
+        new_value //= old_range  # type: ignore
+    else:
+        new_value /= old_range  # type: ignore
+
+    new_value += new_min  # type: ignore
 
     return new_value
